@@ -8,17 +8,17 @@ use std::fmt;
 use std::io;
 use std::str;
 
+use clap::Parser;
 use console::{Style, Term};
 use dialoguer::{theme, Confirm, Input, MultiSelect, Select};
 use futures::StreamExt;
 use itertools::Itertools;
 use serde::{de, Deserialize, Serialize};
-use structopt::clap;
-use structopt::StructOpt;
+use serde_json as json;
 
 use crate::api;
 use crate::k8s;
-use crate::location::Location;
+use crate::location::{AwsRegion, AzureRegion, Location};
 use crate::show::{Detailed, Quiet};
 use crate::traits::Show;
 use crate::v0;
@@ -39,10 +39,10 @@ mod print;
 
 const ABOUT: &str = "statehub CLI tool";
 
-#[derive(Debug, StructOpt)]
-#[structopt(about = ABOUT)]
+#[derive(Debug, clap::Parser)]
+#[clap(about = ABOUT)]
 pub(crate) struct Cli {
-    #[structopt(
+    #[clap(
         help = "Management server URL or address",
         // default_value = "https://api.statehub.io",
         long,
@@ -51,7 +51,7 @@ pub(crate) struct Cli {
         env = "SHAPI"
     )]
     management: Option<String>,
-    #[structopt(
+    #[clap(
         help = "Console server URL or address",
         // default_value = "https://console.statehub.io",
         long,
@@ -59,117 +59,117 @@ pub(crate) struct Cli {
         env = "SHCONSOLE"
     )]
     console: Option<String>,
-    #[structopt(help = "Authentication token", long, global = true, env = "SHTOKEN")]
+    #[clap(help = "Authentication token", long, global = true, env = "SHTOKEN")]
     token: Option<String>,
-    #[structopt(long, global = true, help = "Show results as JSON")]
+    #[clap(long, global = true, help = "Show results as JSON")]
     json: bool,
-    #[structopt(
+    #[clap(
         help = "Configuration profile to use",
         long,
         global = true,
         env = "SHPROFILE"
     )]
     profile: Option<String>,
-    #[structopt(short, long, global = true)]
+    #[clap(short, long, global = true)]
     verbose: bool,
-    #[structopt(subcommand)]
+    #[clap(subcommand)]
     command: Command,
 }
 
-#[derive(Debug, PartialEq, StructOpt)]
+#[derive(Debug, PartialEq, clap::Subcommand)]
 enum Command {
-    #[structopt(about = "Authenticate against statehub service", display_order(0))]
+    #[clap(about = "Authenticate against statehub service", display_order(0))]
     Login,
 
-    #[structopt(about = "Get entity info")]
+    #[clap(about = "Get entity info")]
     Get { entity: Entity },
 
-    #[structopt(
+    #[clap(
         about = "Create new state",
         visible_aliases = &["create-st", "cs"],
         display_order(20),
     )]
     CreateState {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         name: v0::StateName,
-        #[structopt(long, short, help = "Default owning cluster")]
+        #[clap(long, short, help = "Default owning cluster")]
         owner: Option<v0::ClusterName>,
-        #[structopt(long, short, help = "Location definition", alias = "location")]
+        #[clap(long, short, help = "Location definition", alias = "location")]
         locations: Vec<Location>,
-        #[structopt(
+        #[clap(
             long,
             short,
             help = "Get locations from this cluster",
             alias = "cluster"
         )]
         clusters: Vec<v0::ClusterName>,
-        #[structopt(
+        #[clap(
             long,
             help = "Storage class name if different from the state name",
             visible_alias = "sc"
         )]
         storage_class: Option<String>,
-        #[structopt(
+        #[clap(
             long,
             help = "Default filesystem for volumes of this state",
             visible_alias = "fs"
         )]
         fs_type: Option<v0::VolumeFileSystem>,
-        #[structopt(
+        #[clap(
             long,
             help = "Extra query parameters for create_state API call",
             parse(try_from_str = parse_params),
             alias = "param",
-            hidden = true,
+            hide = true,
         )]
         params: Vec<(String, String)>,
     },
 
-    #[structopt(
+    #[clap(
         about = "Delete existing state",
         visible_aliases = &["delete-st", "ds"],
         display_order(20),
     )]
     DeleteState {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         name: v0::StateName,
     },
 
-    #[structopt(
+    #[clap(
         about = "List available states",
         visible_aliases = &["list-state", "list-st", "ls"],
         display_order(20),
     )]
     ListStates,
 
-    #[structopt(
+    #[clap(
         about = "Show state details",
         visible_aliases = &["show-s", "ss"],
         display_order(20),
     )]
     ShowState {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         name: v0::StateName,
     },
 
-    #[structopt(
+    #[clap(
         about = "Register new cluster",
         visible_aliases = &["register-cl", "rc"],
         display_order(10),
     )]
     RegisterCluster {
-        #[structopt(
+        #[clap(
             help = "Non-interactive cluster registration, automatic defaults apply",
             long,
-            short = "y",
+            short = 'y',
             visible_alias = "yes"
         )]
         non_interactive: bool,
 
-        #[structopt(help = "Cluster name, defaults to current k8s context")]
+        #[clap(help = "Cluster name, defaults to current k8s context")]
         names: Vec<v0::ClusterName>,
 
-        #[structopt(
+        #[clap(
             help = "Kubeconfig context to use for registering cluster",
             long,
             short,
@@ -177,7 +177,7 @@ enum Command {
         )]
         contexts: Vec<String>,
 
-        #[structopt(
+        #[clap(
             help = "List of states to for this cluster to use",
             long,
             visible_alias = "state",
@@ -185,21 +185,21 @@ enum Command {
         )]
         states: Vec<v0::StateName>,
 
-        #[structopt(
+        #[clap(
             help = "Skip adding this cluster locations to any state",
             long,
             conflicts_with = "states"
         )]
         no_state: bool,
 
-        #[structopt(
+        #[clap(
             help = "Skip setting up default storage class",
             long,
             conflicts_with = "default_storage_class"
         )]
         no_default_storage_class: bool,
 
-        #[structopt(
+        #[clap(
             help = "The name of the state to configure as default storage class",
             long,
             conflicts_with_all = &["no_state", "no_default_storage_class"],
@@ -207,10 +207,10 @@ enum Command {
         )]
         default_storage_class: Option<String>,
 
-        #[structopt(help = "Do not register this cluster as state owner", long)]
+        #[clap(help = "Do not register this cluster as state owner", long)]
         no_state_owner: bool,
 
-        #[structopt(
+        #[clap(
             help = "Namespace to install statehub components",
             long,
             default_value = "statehub-system",
@@ -218,32 +218,32 @@ enum Command {
         )]
         namespace: String,
 
-        #[structopt(help = "Skip running 'helm install'", long)]
+        #[clap(help = "Skip running 'helm install'", long)]
         skip_helm: bool,
 
-        #[structopt(help = "K8s cluster provider [default: autodetect]", long)]
+        #[clap(help = "K8s cluster provider [default: autodetect]", long)]
         provider: Option<v0::Provider>,
 
-        #[structopt(help = "Cleanup grace time out", long, default_value = "600s")]
+        #[clap(help = "Cleanup grace time out", long, default_value = "600s")]
         cleanup_grace: String,
 
-        #[structopt(flatten)]
+        #[clap(flatten)]
         aws: AwsCredentials,
 
-        #[structopt(flatten)]
+        #[clap(flatten)]
         azure: AzureCredentials,
     },
 
-    #[structopt(
+    #[clap(
         about = "Unregister existing cluster",
         visible_aliases = &["unregister-cl", "urc", "uc"],
         display_order(11),
     )]
     UnregisterCluster {
-        #[structopt(help = "Skip confirmation", long, short)]
+        #[clap(help = "Skip confirmation", long, short)]
         force: bool,
 
-        #[structopt(
+        #[clap(
             help = "Kubeconfig context to use for unregistering cluster",
             long,
             short,
@@ -251,38 +251,38 @@ enum Command {
         )]
         context: Option<String>,
 
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         name: Option<v0::ClusterName>,
 
-        #[structopt(
+        #[clap(
             help = "Non-interactive cluster unregistration, automatic defaults apply",
             long,
-            short = "y",
+            short = 'y',
             alias = "yes",
-            hidden = true
+            hide = true
         )]
         non_interactive: bool,
     },
 
-    #[structopt(
+    #[clap(
         about = "Update registered cluster",
         visible_aliases = &["update-cl", "upc"],
         display_order(12),
     )]
     UpdateCluster {
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         name: Option<v0::ClusterName>,
-        #[structopt(
+        #[clap(
             help = "Namespace where statehub components are installed",
             long,
             default_value = "statehub-system"
         )]
         namespace: String,
-        #[structopt(flatten)]
+        #[clap(flatten)]
         aws: AwsCredentials,
-        #[structopt(flatten)]
+        #[clap(flatten)]
         azure: AzureCredentials,
-        #[structopt(
+        #[clap(
             help = "Kubeconfig context to use for updating cluster",
             long,
             short,
@@ -291,30 +291,30 @@ enum Command {
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "List registered clusters",
         visible_aliases = &["list-cluster", "list-cl", "lc"],
         display_order(13),
     )]
     ListClusters,
 
-    #[structopt(
+    #[clap(
         about = "Show registered cluster details",
         visible_aliases = &["show-cl", "sc"],
         display_order(14),
     )]
     ShowCluster {
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         name: Option<v0::ClusterName>,
     },
 
-    #[structopt(
+    #[clap(
         about = "Diagnose registered cluster for any irregularities",
         visible_aliases = &["diagnose-cl", "dc"],
         display_order(15),
     )]
     DiagnoseCluster {
-        #[structopt(
+        #[clap(
             help = "Kubeconfig context to use for installing Statehub Cluster CRD",
             long,
             short
@@ -322,147 +322,147 @@ enum Command {
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "Manually make state available in a specified location",
         visible_aliases = &["add-l", "al"],
         display_order(30),
     )]
     AddLocation {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(
+        #[clap(
             help = "Location specification",
             conflicts_with = "cluster",
-            required_unless = "cluster"
+            required_unless_present = "cluster"
         )]
         location: Option<Location>,
-        #[structopt(
+        #[clap(
             help = "Add locations from this cluster",
             long,
             short,
             conflicts_with = "location",
-            required_unless = "location"
+            required_unless_present = "location"
         )]
         cluster: Option<v0::ClusterName>,
-        #[structopt(help = "Wait until new location is ready", long)]
+        #[clap(help = "Wait until new location is ready", long)]
         wait: bool,
     },
 
-    #[structopt(
+    #[clap(
         about = "Manually make state unavailable in a specified location",
         visible_aliases = &["remove-l", "rem-l", "rl"],
         display_order(30),
     )]
     RemoveLocation {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Location specification")]
+        #[clap(help = "Location specification")]
         location: Location,
     },
 
-    #[structopt(
+    #[clap(
         about = "Set state availability grade",
         display_order(40),
         setting(clap::AppSettings::Hidden)
     )]
     SetAvailability,
 
-    #[structopt(about = "Set cluster as the state owner", display_order(40))]
+    #[clap(about = "Set cluster as the state owner", display_order(40))]
     SetOwner {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         cluster: v0::ClusterName,
     },
 
-    #[structopt(about = "Clear state owner", display_order(40))]
+    #[clap(about = "Clear state owner", display_order(40))]
     UnsetOwner {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         cluster: v0::ClusterName,
     },
 
-    #[structopt(
+    #[clap(
         about = "Manually create new volume",
         visible_aliases = &["create-v", "cv"],
         display_order(50),
         setting(clap::AppSettings::Hidden),
     )]
     CreateVolume {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Volume name")]
+        #[clap(help = "Volume name")]
         volume: v0::VolumeName,
-        #[structopt(help = "valume size")]
+        #[clap(help = "valume size")]
         size: u64,
-        #[structopt(help = "volume file system")]
+        #[clap(help = "volume file system")]
         fs_type: v0::VolumeFileSystem,
     },
 
-    #[structopt(
+    #[clap(
         about = "Manually delete existing volume",
         visible_aliases = &["delete-v", "dv"],
         display_order(50),
         setting(clap::AppSettings::Hidden),
     )]
     DeleteVolume {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Volume name")]
+        #[clap(help = "Volume name")]
         volume: v0::VolumeName,
-        #[structopt(help = "Wait until volume is deleted", long)]
+        #[clap(help = "Wait until volume is deleted", long)]
         wait: bool,
     },
 
-    #[structopt(
+    #[clap(
         about = "Manually choose primary location for volume",
         visible_aliases = &["set-p", "sp"],
         display_order(50),
         setting(clap::AppSettings::Hidden),
     )]
     SetPrimary {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Volume name")]
+        #[clap(help = "Volume name")]
         volume: v0::VolumeName,
-        #[structopt(help = "Primary location specification", long, short)]
+        #[clap(help = "Primary location specification", long, short)]
         primary: Location,
     },
 
-    #[structopt(
+    #[clap(
         about = "List volumes in the given state",
         visible_aliases = &["list-volume", "list-v", "lv"],
         display_order(50),
     )]
     ListVolumes {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
     },
 
-    #[structopt(
+    #[clap(
         about = "Show volume[s] info in the given state",
         visible_aliases = &["show-volume", "show-v", "sv"],
         display_order(50),
     )]
     ShowVolumes {
-        #[structopt(help = "State name")]
+        #[clap(help = "State name")]
         state: v0::StateName,
-        #[structopt(help = "Volume name")]
+        #[clap(help = "Volume name")]
         volumes: Vec<v0::VolumeName>,
     },
 
-    #[structopt(
+    #[clap(
         about = "Uninstall statehub objects from the cluster",
         visible_aliases = &["uninstall-sh", "us"],
         display_order(50)
     )]
     UninstallStatehub {
-        #[structopt(help = "Skip confirmation", long, short)]
+        #[clap(help = "Skip confirmation", long, short)]
         force: bool,
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         name: v0::ClusterName,
-        #[structopt(
+        #[clap(
             help = "Kubeconfig context to use for uninstalling statehub",
             long,
             short,
@@ -471,113 +471,113 @@ enum Command {
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "Just register named cluster via API (no interaction with K8s cluster",
         visible_alias = "jrc",
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
     JustRegisterCluster {
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         name: v0::ClusterName,
-        #[structopt(
+        #[clap(
             help = "Namespace to install statehub components",
             long,
             default_value = "statehub-system",
             visible_alias = "ns"
         )]
         namespace: String,
-        #[structopt(help = "K8s cluster provider", long, default_value = "generic")]
+        #[clap(help = "K8s cluster provider", long, default_value = "generic")]
         provider: v0::Provider,
-        #[structopt(long, short, help = "Location definition", alias = "location")]
+        #[clap(long, short, help = "Location definition", alias = "location")]
         locations: Vec<Location>,
     },
 
-    #[structopt(
+    #[clap(
         about = "Just unregister named cluster via API (no interaction with K8s cluster)",
         visible_aliases = &["jurc", "juc"],
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
     JustUnregisterCluster {
-        #[structopt(help = "Cluster name")]
+        #[clap(help = "Cluster name")]
         name: v0::ClusterName,
     },
 
-    #[structopt(
+    #[clap(
         about = "Save cluster token",
         visible_alias = "sct",
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
     SaveClusterToken {
-        #[structopt(help = "Namespace name")]
+        #[clap(help = "Namespace name")]
         namespace: String,
-        #[structopt(help = "Cluster token")]
+        #[clap(help = "Cluster token")]
         cluster_token: String,
-        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "Setup statehub configmap",
         visible_aliases = &["scm", "setup-cm"],
         display_order(1000),
         setting(clap::AppSettings::Hidden),
     )]
     SetupConfigmap {
-        #[structopt(help = "cluster name")]
+        #[clap(help = "cluster name")]
         cluster: v0::ClusterName,
-        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
-        #[structopt(
+        #[clap(
             help = "Namespace to install statehub components",
             default_value = "statehub-system"
         )]
         namespace: String,
-        #[structopt(
+        #[clap(
             help = "The name of the state to configure as default storage class, nothing by default"
         )]
         default_state: Option<String>,
-        #[structopt(
+        #[clap(
             help = "The name of the state to configure as default storage class, nothing by default",
             default_value = "600s"
         )]
         cleanup_grace: String,
     },
 
-    #[structopt(
+    #[clap(
         about = "Install Statehub Cluster CRD",
         visible_aliases = &["icc", "crd"],
         display_order(1000),
         setting(clap::AppSettings::Hidden),
     )]
     InstallClusterCrd {
-        #[structopt(help = "cluster name")]
+        #[clap(help = "cluster name")]
         cluster: v0::ClusterName,
-        #[structopt(
+        #[clap(
             help = "Kubeconfig context to use for installing Statehub Cluster CRD",
             long,
             short
         )]
         context: Option<String>,
-        #[structopt(
+        #[clap(
             help = "Namespace to install statehub components",
             default_value = "statehub-system"
         )]
         namespace: String,
     },
 
-    #[structopt(
+    #[clap(
         about = "Uninstall Statehub Cluster CRD",
         visible_aliases = &["ucc", "uncrd"],
         display_order(1000),
         setting(clap::AppSettings::Hidden),
     )]
     UninstallClusterCrd {
-        #[structopt(help = "cluster name")]
+        #[clap(help = "cluster name")]
         cluster: v0::ClusterName,
-        #[structopt(
+        #[clap(
             help = "Kubeconfig context to use for uninstalling Statehub Cluster CRD",
             long,
             short
@@ -585,57 +585,57 @@ enum Command {
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "List K8s namespaces",
         visible_alias = "list-ns",
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
     ListNamespaces {
-        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "List K8s nodes",
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
     ListNodes {
-        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "List K8s pods",
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
     ListPods {
-        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
     },
 
-    #[structopt(
+    #[clap(
         about = "List K8s node regions",
         visible_aliases = &["list-r", "lr"],
         display_order(1000),
         setting(clap::AppSettings::Hidden),
     )]
     ListRegions {
-        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        #[clap(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
         context: Option<String>,
-        #[structopt(help = "Also list zones", long, short)]
+        #[clap(help = "Also list zones", long, short)]
         zone: bool,
     },
 
-    #[structopt(about = "Save default configuration file", display_order(2000))]
+    #[clap(about = "Save default configuration file", display_order(2000))]
     SaveConfig {
-        #[structopt(help = "Save selected profile as default profile", long, short = "d")]
+        #[clap(help = "Save selected profile as default profile", long, short = 'd')]
         set_default_profile: bool,
     },
 
-    #[structopt(
+    #[clap(
         about = "Check for updates to statehub utility",
         visible_alias = "up",
         display_order(3000)
@@ -693,7 +693,12 @@ impl Command {
 
 impl Cli {
     pub(crate) async fn execute() -> anyhow::Result<()> {
-        Self::from_args().dispatch().await
+        let cli = Self::parse();
+        let json = cli.json;
+
+        cli.dispatch()
+            .await
+            .or_else(|err| if json { json_error(err) } else { Err(err) })
     }
 
     async fn config(&self) -> anyhow::Result<Config> {
@@ -1647,3 +1652,12 @@ fn parse_params(text: &str) -> anyhow::Result<(String, String)> {
         .map(|(key, value)| (key.to_string(), value.to_string()))
         .ok_or_else(|| anyhow::anyhow!("Invalid format: should be key=value"))
 }
+
+fn json_error(err: anyhow::Error) -> anyhow::Result<()> {
+    if let Some(err) = err.downcast_ref::<v0::Error>() {
+        println!("{}", json::to_string(err)?);
+        Ok(())
+    } else {
+        Err(err)
+    }
+}
