@@ -3,19 +3,19 @@
 // Use is subject to license terms.
 //
 
+use clap::Parser;
 use serde::{Deserialize, Serialize};
 use serde_with::skip_serializing_none;
-use structopt::StructOpt;
 
 #[skip_serializing_none]
-#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, StructOpt)]
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Parser)]
 #[serde(rename_all = "UPPERCASE")]
 pub(crate) struct AwsCredentials {
-    // #[structopt(help = "AWS Access Key ID", long, requires = "aws_secret_access_key")]
-    #[structopt(help = "AWS Access Key ID", long)]
+    // #[clap(help = "AWS Access Key ID", long, requires = "aws_secret_access_key")]
+    #[clap(help = "AWS Access Key ID", long)]
     aws_access_key_id: Option<String>,
-    // #[structopt(help = "AWS Secret Access Key", long, requires = "aws_access_key_id")]
-    #[structopt(help = "AWS Secret Access Key", long)]
+    // #[clap(help = "AWS Secret Access Key", long, requires = "aws_access_key_id")]
+    #[clap(help = "AWS Secret Access Key", long)]
     aws_secret_access_key: Option<String>,
 }
 
