@@ -407,7 +407,7 @@ impl StateHub {
     async fn add_aws_location_helper(
         &self,
         name: &v0::StateName,
-        region: v0::AwsRegion,
+        region: AwsRegion,
         wait: bool,
     ) -> anyhow::Result<Output<v0::StateLocationAws>> {
         let aws = self.api.add_aws_location(name, region).await?;
@@ -432,7 +432,7 @@ impl StateHub {
     async fn add_azure_location_helper(
         &self,
         name: &v0::StateName,
-        region: v0::AzureRegion,
+        region: AzureRegion,
         wait: bool,
     ) -> anyhow::Result<Output<v0::StateLocationAzure>> {
         let azure = self.api.add_azure_location(name, region).await?;
@@ -812,25 +812,25 @@ mod tests {
     #[test]
     fn generate_add_location_commands() {
         let missing_locations = vec![
-            (state("alfa"), vec![Location::Aws(v0::AwsRegion::UsWest2)]),
+            (state("alfa"), vec![Location::Aws(AwsRegion::UsWest2)]),
             (
                 state("bravo"),
                 vec![
-                    Location::Aws(v0::AwsRegion::UsEast1),
-                    Location::Azure(v0::AzureRegion::EastUs2),
+                    Location::Aws(AwsRegion::UsEast1),
+                    Location::Azure(AzureRegion::EastUs2),
                 ],
             ),
-            (
-                state("charlie"),
-                vec![Location::Aws(v0::AwsRegion::UsWest2)],
-            ),
+            (state("charlie"), vec![Location::Aws(AwsRegion::UsWest2)]),
         ]
         .into_iter()
         .collect();
         let cmds = super::generate_add_location_commands(&missing_locations);
-        assert_eq!(cmds[0], "statehub add-location --wait alfa us-west-2");
-        assert_eq!(cmds[1], "statehub add-location --wait bravo us-east-1");
-        assert_eq!(cmds[2], "statehub add-location --wait bravo eastus2");
-        assert_eq!(cmds[3], "statehub add-location --wait charlie us-west-2");
+        assert_eq!(cmds[0], "statehub add-location --wait alfa aws/us-west-2");
+        assert_eq!(cmds[1], "statehub add-location --wait bravo aws/us-east-1");
+        assert_eq!(cmds[2], "statehub add-location --wait bravo azure/eastus2");
+        assert_eq!(
+            cmds[3],
+            "statehub add-location --wait charlie aws/us-west-2"
+        );
     }
 }
